@@ -36,6 +36,7 @@ from cmdstanpy.utils import (
     MaybeDictToFilePath,
     TemporaryCopiedFile,
     get_logger,
+    scan_sampler_csv,
 )
 
 
@@ -126,7 +127,7 @@ class CmdStanModel:
 
         if platform.system() == 'Windows':
             # Add tbb to the $PATH on Windows
-            libtbb = os.getenv('STAN_TBB')
+            libtbb = os.environ.get('STAN_TBB')
             if libtbb is None:
                 libtbb = os.path.join(
                     cmdstan_path(), 'stan', 'lib', 'stan_math', 'lib', 'tbb'
@@ -134,7 +135,7 @@ class CmdStanModel:
             os.environ['PATH'] = ';'.join(
                 list(
                     OrderedDict.fromkeys(
-                        [libtbb] + os.getenv('PATH', '').split(';')
+                        [libtbb] + os.environ.get('PATH', '').split(';')
                     )
                 )
             )
@@ -208,7 +209,7 @@ class CmdStanModel:
 
         By default, this function compares the timestamps on the source and
         executable files; if the executable is newer than the source file, it
-        will not recompile the file, unless argument ``force`` is True.
+        will not recompile the file, unless argument ``force`` is ``True``.
 
         :param force: When ``True``, always compile, even if the executable file
             is newer than the source file.  Used for Stan models which have
@@ -320,45 +321,46 @@ class CmdStanModel:
         The output files are written either to a specified output directory
         or to a temporary directory which is deleted upon session exit.
 
-        Output filenames are composed of the model name, a timestamp
-        in the form YYYYMMDDhhmm and the chain id, plus the corresponding
-        filetype suffix, either '.csv' for the CmdStan output or '.txt' for
-        the console messages, e.g. `bernoulli-201912081451-1.csv`. Output files
-        written to the temporary directory contain an additional 8-character
-        random string, e.g. `bernoulli-201912081451-1-5nm6as7u.csv`.
+        Output files are either written to a temporary directory or to the
+        specified output directory.  Ouput filenames correspond to the template
+        '<model_name>-<YYYYMMDDHHMM>-<chain_id>' plus the file suffix which is
+        either '.csv' for the CmdStan output or '.txt' for
+        the console messages, e.g. 'bernoulli-201912081451-1.csv'.
+        Output files written to the temporary directory contain an additional
+        8-character random string, e.g. 'bernoulli-201912081451-1-5nm6as7u.csv'.
 
         :param data: Values for all data variables in the model, specified
             either as a dictionary with entries matching the data variables,
             or as the path of a data file in JSON or Rdump format.
 
         :param seed: The seed for random number generator. Must be an integer
-            between ``0`` and ``2^32 - 1``. If unspecified,
-            ``numpy.random.RandomState()``
-            is used to generate a seed which will be used for all chains.
+            between 0 and 2^32 - 1. If unspecified,
+            ``numpy.random.RandomState()`` is used to generate a seed.
 
         :param inits:  Specifies how the sampler initializes parameter values.
             Initialization is either uniform random on a range centered on 0,
             exactly 0, or a dictionary or file of initial values for some or
             all parameters in the model.  The default initialization behavior
             will initialize all parameter values on range [-2, 2] on the
-            _unconstrained_ support.  If the expected parameter values are
+            *unconstrained* support.  If the expected parameter values are
             too far from this range, this option may improve estimation.
             The following value types are allowed:
 
-            * Single number ``n > 0`` - initialization range is [-n, n].
-            * ``0`` - all parameters are initialized to 0.
+            * Single number, n > 0 - initialization range is [-n, n].
+            * 0 - all parameters are initialized to 0.
             * dictionary - pairs parameter name : initial value.
             * string - pathname to a JSON or Rdump data file.
 
-        :param output_dir:  Name of the directory in which the CmdStan output
-            files are saved.  If unspecified, files will be written to a
-            temporary directory which is deleted upon session exit.
+        :param output_dir: Name of the directory to which CmdStan output
+            files are written. If unspecified, output files will be written
+            to a temporary directory which is deleted upon session exit.
 
         :param save_diagnostics: Whether or not to save diagnostics. If True,
-            csv output files are written to ``<basename>-diagnostic-0.csv.``,
-            where ``<basename>`` is set with ``csv_basename``.
+            csv output files are written to an output file with filename
+            template '<model_name>-<YYYYMMDDHHMM>-diagnostic-<chain_id>',
+            e.g. 'bernoulli-201912081451-diagnostic-1.csv'.
 
-        :param algorithm: Algorithm to use. One of: "BFGS", "LBFGS", "Newton"
+        :param algorithm: Algorithm to use. One of: 'BFGS', 'LBFGS', 'Newton'
 
         :param init_alpha: Line search step size for first iteration
 
@@ -395,14 +397,14 @@ class CmdStanModel:
                 )
                 raise RuntimeError(msg)
         mle = CmdStanMLE(runset)
-        mle._set_mle_attrs(runset.csv_files[0])
         return mle
 
     def sample(
         self,
         data: Union[Dict, str] = None,
         chains: Union[int, None] = None,
-        cores: Union[int, None] = None,
+        parallel_chains: Union[int, None] = None,
+        threads_per_chain: Union[int, None] = None,
         seed: Union[int, List[int]] = None,
         chain_ids: Union[int, List[int]] = None,
         inits: Union[Dict, float, str, List[str]] = None,
@@ -415,6 +417,9 @@ class CmdStanModel:
         step_size: Union[float, List[float]] = None,
         adapt_engaged: bool = True,
         adapt_delta: float = None,
+        adapt_init_phase: int = None,
+        adapt_metric_window: int = None,
+        adapt_step_size: int = None,
         fixed_param: bool = False,
         output_dir: str = None,
         save_diagnostics: bool = False,
@@ -436,27 +441,32 @@ class CmdStanModel:
         specified output directory or to a temporary directory which is deleted
         upon session exit.
 
-        The output filenames are composed of the model name, a timestamp
-        in the form YYYYMMDDhhmm and the chain id, plus the corresponding
-        filetype suffix, either '.csv' for the CmdStan output or '.txt' for
-        the console messages, e.g. `bernoulli-201912081451-1.csv`. Output files
-        written to the temporary directory contain an additional 8-character
-        random string, e.g. `bernoulli-201912081451-1-5nm6as7u.csv`.
-
+        Output files are either written to a temporary directory or to the
+        specified output directory.  Ouput filenames correspond to the template
+        '<model_name>-<YYYYMMDDHHMM>-<chain_id>' plus the file suffix which is
+        either '.csv' for the CmdStan output or '.txt' for
+        the console messages, e.g. 'bernoulli-201912081451-1.csv'.
+        Output files written to the temporary directory contain an additional
+        8-character random string, e.g. 'bernoulli-201912081451-1-5nm6as7u.csv'.
 
         :param data: Values for all data variables in the model, specified
             either as a dictionary with entries matching the data variables,
             or as the path of a data file in JSON or Rdump format.
 
-        :param chains: Number of sampler chains, should be > 1.
+        :param chains: Number of sampler chains, must be a positive integer.
 
-        :param cores: Number of processes to run in parallel. Must be an
-            integer between 1 and the number of CPUs in the system.
-            If none then set automatically to `chains` but no more
-            than `total_cpu_count - 2`
+        :param parallel_chains: Number of processes to run in parallel. Must be
+            a positive integer.  Defaults to ``multiprocessing.cpu_count()``.
+
+        :threads_per_chain: If the model was compiled with threading support,
+            the number of threads to use in parallelized sections within an
+            MCMC chain (e.g., when using the Stan functions ``reduce_sum()``
+            or ``map_rect()``).  The total number of threads used will be
+            ``parallel_chains * threads_per_chain``.  If a model wasn't
+            compiled with threading support, has no effect.
 
         :param seed: The seed for random number generator. Must be an integer
-            between ``0`` and ``2^32 - 1``. If unspecified,
+            between 0 and 2^32 - 1. If unspecified,
             ``numpy.random.RandomState()``
             is used to generate a seed which will be used for all chains.
             When the same seed is used across all chains,
@@ -471,12 +481,12 @@ class CmdStanModel:
             exactly 0, or a dictionary or file of initial values for some or all
             parameters in the model.  The default initialization behavior will
             initialize all parameter values on range [-2, 2] on the
-            _unconstrained_ support.  If the expected parameter values are
+            *unconstrained* support.  If the expected parameter values are
             too far from this range, this option may improve adaptation.
             The following value types are allowed:
 
-            * Single number ``n > 0`` - initialization range is [-n, n].
-            * ``0`` - all parameters are initialized to 0.
+            * Single number n > 0 - initialization range is [-n, n].
+            * 0 - all parameters are initialized to 0.
             * dictionary - pairs parameter name : initial value.
             * string - pathname to a JSON or Rdump data file.
             * list of strings - per-chain pathname to data file.
@@ -486,7 +496,7 @@ class CmdStanModel:
         :param iter_sampling: Number of draws from the posterior for each
             chain.
 
-        :param save_warmup: When True, sampler saves warmup draws as part of
+        :param save_warmup: When ``True``, sampler saves warmup draws as part of
             the Stan csv output file.
 
         :param thin: Period between saved samples.
@@ -496,13 +506,13 @@ class CmdStanModel:
 
         :param metric: Specification of the mass matrix, either as a
             vector consisting of the diagonal elements of the covariance
-            matrix (``diag`` or ``diag_e``) or the full covariance matrix
-            (``dense`` or ``dense_e``).
+            matrix ('diag' or 'diag_e') or the full covariance matrix
+            ('dense' or 'dense_e').
 
             If the value of the metric argument is a string other than
-            ``diag``, ``diag_e``, ``dense``, or ``dense_e``, it must be
+            'diag', 'diag_e', 'dense', or 'dense_e', it must be
             a valid filepath to a JSON or Rdump file which contains an entry
-            ``inv_metric`` whose value is either the diagonal vector or
+            'inv_metric' whose value is either the diagonal vector or
             the full covariance matrix.
 
             If the value of the metric argument is a list of paths, its
@@ -511,37 +521,49 @@ class CmdStanModel:
 
         :param step_size: Initial stepsize for HMC sampler.  The value is either
             a single number or a list of numbers which will be used as the
-            global or per-chain initial step_size, respectively.
+            global or per-chain initial step size, respectively.
             The length of the list of step sizes must match the number of
             chains.
 
         :param adapt_engaged: When True, adapt stepsize and metric.
-            *Note: If True, ``iter_warmup`` must be > 0.*
 
         :param adapt_delta: Adaptation target Metropolis acceptance rate.
             The default value is 0.8.  Increasing this value, which must be
-            strictly less than 1, causes adaptation to use smaller step sizes.
-            It improves the effective sample size, but may increase the time
+            strictly less than 1, causes adaptation to use smaller step sizes
+            which improves the effective sample size, but may increase the time
             per iteration.
 
-        :param fixed_param: When True, call CmdStan with argument
-            "algorithm=fixed_param" which runs the sampler without
+        :param adapt_init_phase: Iterations for initial phase of adaptation
+            during which step size is adjusted so that the chain converges
+            towards the typical set.
+
+        :param adapt_metric_window: The second phase of adaptation tunes
+            the metric and stepsize in a series of intervals.  This parameter
+            specifies the number of iterations used for the first tuning
+            interval; window size increases for each subsequent interval.
+
+        :param adapt_step_size: Number of iterations given over to adjusting
+            the step size given the tuned metric during the final phase of
+            adaptation.
+
+        :param fixed_param: When ``True``, call CmdStan with argument
+            ``algorithm=fixed_param`` which runs the sampler without
             updating the Markov Chain, thus the values of all parameters and
             transformed parameters are constant across all draws and
             only those values in the generated quantities block that are
             produced by RNG functions may change.  This provides
             a way to use Stan programs to generate simulated data via the
             generated quantities block.  This option must be used when the
-            parameters block is empty.  Default value is False.
+            parameters block is empty.  Default value is ``False``.
 
-        :param output_dir: Name of the directory to with the  CmdStan output
+        :param output_dir: Name of the directory to which CmdStan output
             files are written. If unspecified, output files will be written
             to a temporary directory which is deleted upon session exit.
 
         :param save_diagnostics: Whether or not to save diagnostics. If True,
-            csv output files are written to
-            ``<basename>-diagnostic-<chain_id>.csv.``, where ``<basename>``
-            is set with ``csv_basename``.
+            csv output files are written to an output file with filename
+            template '<model_name>-<YYYYMMDDHHMM>-diagnostic-<chain_id>',
+            e.g. 'bernoulli-201912081451-diagnostic-1.csv'.
 
         :param show_progress: Use tqdm progress bar to show sampling progress.
             If show_progress=='notebook' use tqdm_notebook
@@ -549,7 +571,6 @@ class CmdStanModel:
 
         :return: CmdStanMCMC object
         """
-
         if chains is None:
             if fixed_param:
                 chains = 1
@@ -561,18 +582,16 @@ class CmdStanModel:
                     chains
                 )
             )
-
         if chain_ids is None:
             chain_ids = [x + 1 for x in range(chains)]
         else:
             if isinstance(chain_ids, int):
-                if chain_ids < 1:
+                if chain_ids < 0:
                     raise ValueError(
-                        'Chain_id must be a positive integer value,'
+                        'Chain_id must be a non-negative integer value,'
                         ' found {}.'.format(chain_ids)
                     )
-                offset = chain_ids
-                chain_ids = [x + offset + 1 for x in range(chains)]
+                chain_ids = [chain_ids + i + 1 for i in range(chains)]
             else:
                 if not len(chain_ids) == chains:
                     raise ValueError(
@@ -581,26 +600,38 @@ class CmdStanModel:
                             chains, len(chain_ids)
                         )
                     )
-                for i in len(chain_ids):
-                    if chain_ids[i] < 1:
+                for chain_id in chain_ids:
+                    if chain_id < 0:
                         raise ValueError(
-                            'Chain_id must be a positive integer value,'
-                            ' found {}.'.format(chain_ids[i])
+                            'Chain_id must be a non-negative integer value,'
+                            ' found {}.'.format(chain_id)
                         )
-
-        cores_avail = cpu_count()
-        if cores is None:
-            cores = max(min(cores_avail - 2, chains), 1)
-        if cores < 1:
-            raise ValueError(
-                'Argument cores must be a positive integer value, '
-                'found {}.'.format(cores)
-            )
-        if cores > cores_avail:
+        if parallel_chains is None:
+            parallel_chains = max(min(cpu_count(), chains), 1)
+        elif parallel_chains > chains:
             self._logger.warning(
-                'Requested %u cores, only %u available.', cores, cpu_count()
+                'Requesting %u parallel_chains for %u chains,'
+                ' running all chains in parallel.',
+                parallel_chains,
+                chains,
             )
-            cores = cores_avail
+            parallel_chains = chains
+        elif parallel_chains < 1:
+            raise ValueError(
+                'Argument parallel_chains must be a positive integer value, '
+                'found {}.'.format(parallel_chains)
+            )
+        if threads_per_chain is None:
+            threads_per_chain = 1
+        if threads_per_chain < 1:
+            raise ValueError(
+                'Argument threads_per_chain must be a positive integer value, '
+                'found {}.'.format(threads_per_chain)
+            )
+        self._logger.debug(
+            'total threads: %u', parallel_chains * threads_per_chain
+        )
+        os.environ['STAN_NUM_THREADS'] = str(threads_per_chain)
 
         refresh = None
         if show_progress:
@@ -630,6 +661,9 @@ class CmdStanModel:
             step_size=step_size,
             adapt_engaged=adapt_engaged,
             adapt_delta=adapt_delta,
+            adapt_init_phase=adapt_init_phase,
+            adapt_metric_window=adapt_metric_window,
+            adapt_step_size=adapt_step_size,
             fixed_param=fixed_param,
         )
         with MaybeDictToFilePath(data, inits) as (_data, _inits):
@@ -644,12 +678,13 @@ class CmdStanModel:
                 save_diagnostics=save_diagnostics,
                 method_args=sampler_args,
                 refresh=refresh,
+                logger=self._logger,
             )
             runset = RunSet(args=args, chains=chains)
             pbar = None
             all_pbars = []
 
-            with ThreadPoolExecutor(max_workers=cores) as executor:
+            with ThreadPoolExecutor(max_workers=parallel_chains) as executor:
                 for i in range(chains):
                     if show_progress:
                         if (
@@ -689,7 +724,6 @@ class CmdStanModel:
                             dynamic_ncols=dynamic_ncols,
                         )
                         all_pbars.append(pbar)
-
                     executor.submit(self._run_cmdstan, runset, i, pbar)
 
             # Closing all progress bars
@@ -704,15 +738,14 @@ class CmdStanModel:
                 for i in range(chains):
                     if runset._retcode(i) != 0:
                         err_msg = '{}chain {} returned error code {}\n'.format(
-                            err_msg, i+1, runset._retcode(i)
+                            err_msg, i + 1, runset._retcode(i)
                         )
                 console_errs = runset._get_err_msgs()
                 if len(console_errs) > 0:
                     err_msg = '{}{}'.format(err_msg, ''.join(console_errs))
                 raise RuntimeError(err_msg)
 
-            mcmc = CmdStanMCMC(runset, fixed_param)
-            mcmc._validate_csv_files()
+            mcmc = CmdStanMCMC(runset)
         return mcmc
 
     def generate_quantities(
@@ -735,24 +768,24 @@ class CmdStanModel:
         The output files are written either to a specified output directory
         or to a temporary directory which is deleted upon session exit.
 
-        Output filenames are composed of the model name, a timestamp
-        in the form YYYYMMDDhhmm and the chain id, plus the corresponding
-        filetype suffix, either '.csv' for the CmdStan output or '.txt' for
-        the console messages, e.g. `bernoulli_ppc-201912081451-1.csv`. Output
-        files  written to the temporary directory contain an additional
-        8-character random string, e.g.
-        `bernoulli_ppc-201912081451-1-5nm6as7u.csv`.
+        Output files are either written to a temporary directory or to the
+        specified output directory.  Output filenames correspond to the template
+        '<model_name>-<YYYYMMDDHHMM>-<chain_id>' plus the file suffix which is
+        either '.csv' for the CmdStan output or '.txt' for
+        the console messages, e.g. 'bernoulli-201912081451-1.csv'.
+        Output files written to the temporary directory contain an additional
+        8-character random string, e.g. 'bernoulli-201912081451-1-5nm6as7u.csv'.
 
         :param data: Values for all data variables in the model, specified
             either as a dictionary with entries matching the data variables,
             or as the path of a data file in JSON or Rdump format.
 
-        :param mcmc_sample: Can be either a CmdStanMCMC object returned by
-            CmdStanPy's `sample` method or a list of stan-csv files generated
+        :param mcmc_sample: Can be either a ``CmdStanMCMC`` object returned by
+            the ``sample`` method or a list of stan-csv files generated
             by fitting the model to the data using any Stan interface.
 
         :param seed: The seed for random number generator. Must be an integer
-            between ``0`` and ``2^32 - 1``. If unspecified,
+            between 0 and 2^32 - 1. If unspecified,
             ``numpy.random.RandomState()``
             is used to generate a seed which will be used for all chains.
             *NOTE: Specifying the seed will guarantee the same result for
@@ -785,7 +818,26 @@ class CmdStanModel:
         try:
             chains = len(sample_csv_files)
             if sample_drawset is None:  # assemble sample from csv files
-                sampler_args = SamplerArgs()
+                config = {}
+                # scan 1st csv file to get config
+                try:
+                    config = scan_sampler_csv(sample_csv_files[0])
+                except ValueError:
+                    config = scan_sampler_csv(sample_csv_files[0], True)
+                conf_iter_sampling = None
+                if 'num_samples' in config:
+                    conf_iter_sampling = int(config['num_samples'])
+                conf_iter_warmup = None
+                if 'num_warmup' in config:
+                    conf_iter_warmup = int(config['num_warmup'])
+                conf_thin = None
+                if 'thin' in config:
+                    conf_thin = int(config['thin'])
+                sampler_args = SamplerArgs(
+                    iter_sampling=conf_iter_sampling,
+                    iter_warmup=conf_iter_warmup,
+                    thin=conf_thin,
+                )
                 args = CmdStanArgs(
                     self._name,
                     self._exe_file,
@@ -795,7 +847,6 @@ class CmdStanModel:
                 runset = RunSet(args=args, chains=chains)
                 runset._csv_files = sample_csv_files
                 sample_fit = CmdStanMCMC(runset)
-                sample_fit._validate_csv_files()
                 sample_drawset = sample_fit.get_drawset()
         except ValueError as e:
             raise ValueError(
@@ -821,9 +872,9 @@ class CmdStanModel:
             )
             runset = RunSet(args=args, chains=chains)
 
-            cores_avail = cpu_count()
-            cores = max(min(cores_avail - 2, chains), 1)
-            with ThreadPoolExecutor(max_workers=cores) as executor:
+            parallel_chains_avail = cpu_count()
+            parallel_chains = max(min(parallel_chains_avail - 2, chains), 1)
+            with ThreadPoolExecutor(max_workers=parallel_chains) as executor:
                 for i in range(chains):
                     executor.submit(self._run_cmdstan, runset, i)
 
@@ -836,7 +887,6 @@ class CmdStanModel:
                         )
                 raise RuntimeError(msg)
             quantities = CmdStanGQ(runset=runset, mcmc_sample=sample_drawset)
-            quantities._set_attrs_gq_csv_files(sample_csv_files[0])
         return quantities
 
     def variational(
@@ -871,37 +921,38 @@ class CmdStanModel:
         The output files are written either to a specified output directory
         or to a temporary directory which is deleted upon session exit.
 
-        Output filenames are composed of the model name, a timestamp
-        in the form YYYYMMDDhhmm and the chain id, plus the corresponding
-        filetype suffix, either '.csv' for the CmdStan output or '.txt' for
-        the console messages, e.g. `bernoulli-201912081451-1.csv`. Output files
-        written to the temporary directory contain an additional 8-character
-        random string, e.g. `bernoulli-201912081451-1-5nm6as7u.csv`.
+        Output files are either written to a temporary directory or to the
+        specified output directory.  Output filenames correspond to the template
+        '<model_name>-<YYYYMMDDHHMM>-<chain_id>' plus the file suffix which is
+        either '.csv' for the CmdStan output or '.txt' for
+        the console messages, e.g. 'bernoulli-201912081451-1.csv'.
+        Output files written to the temporary directory contain an additional
+        8-character random string, e.g. 'bernoulli-201912081451-1-5nm6as7u.csv'.
 
         :param data: Values for all data variables in the model, specified
             either as a dictionary with entries matching the data variables,
             or as the path of a data file in JSON or Rdump format.
 
         :param seed: The seed for random number generator. Must be an integer
-            between ``0`` and ``2^32 - 1``. If unspecified,
+            between 0 and 2^32 - 1. If unspecified,
             ``numpy.random.RandomState()``
             is used to generate a seed which will be used for all chains.
 
         :param inits:  Specifies how the sampler initializes parameter values.
-            Initialization is uniform random on a range centered on ``0`` with
-            default range of ``2``. Specifying a single number ``n > 0`` changes
-            the initialization range to ``[-n, n]``.
+            Initialization is uniform random on a range centered on 0 with
+            default range of 2. Specifying a single number n > 0 changes
+            the initialization range to [-n, n].
 
-        :param output_dir:  Name of the directory in which the CmdStan output
-            files are saved.  If unspecified, files will be written to a
-            temporary directory which is deleted upon session exit.
+        :param output_dir: Name of the directory to which CmdStan output
+            files are written. If unspecified, output files will be written
+            to a temporary directory which is deleted upon session exit.
 
         :param save_diagnostics: Whether or not to save diagnostics. If True,
-            csv output files are written to
-            ``<basename>-diagnostic-<chain_id>.csv.``, where ``<basename>``
-            is set with ``csv_basename``.
+            csv output files are written to an output file with filename
+            template '<model_name>-<YYYYMMDDHHMM>-diagnostic-<chain_id>',
+            e.g. 'bernoulli-201912081451-diagnostic-1.csv'.
 
-        :param algorithm: Algorithm to use. One of: "meanfield", "fullrank".
+        :param algorithm: Algorithm to use. One of: 'meanfield', 'fullrank'.
 
         :param iter: Maximum number of ADVI iterations.
 
@@ -971,7 +1022,6 @@ class CmdStanModel:
                 raise RuntimeError(msg)
         # pylint: disable=invalid-name
         vb = CmdStanVB(runset)
-        vb._set_variational_attrs(runset.csv_files[0])
         return vb
 
     def _run_cmdstan(
@@ -983,6 +1033,9 @@ class CmdStanModel:
         """
         cmd = runset.cmds[idx]
         self._logger.info('start chain %u', idx + 1)
+        self._logger.debug(
+            'threads: %s', str(os.environ.get('STAN_NUM_THREADS'))
+        )
         self._logger.debug('sampling: %s', cmd)
         proc = subprocess.Popen(
             cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ
