@@ -21,12 +21,9 @@ class GenerateQuantitiesTest(unittest.TestCase):
         goodfiles_path = os.path.join(DATAFILES_PATH, 'runset-good', 'bern')
         csv_files = []
         for i in range(4):
-            csv_files.append('{}-{}.csv'.format(goodfiles_path, i+1))
+            csv_files.append('{}-{}.csv'.format(goodfiles_path, i + 1))
 
-        bern_gqs = model.generate_quantities(
-            data=jdata,
-            mcmc_sample=csv_files
-        )
+        bern_gqs = model.generate_quantities(data=jdata, mcmc_sample=csv_files)
         self.assertEqual(
             bern_gqs.runset._args.method, Method.GENERATE_QUANTITIES
         )
@@ -52,9 +49,11 @@ class GenerateQuantitiesTest(unittest.TestCase):
             'y_rep.10',
         ]
         self.assertEqual(bern_gqs.column_names, tuple(column_names))
-        self.assertEqual(bern_gqs.sample_plus_quantities.shape[1],
-                             bern_gqs.mcmc_sample.shape[1] +
-                             bern_gqs.generated_quantities_pd.shape[1])
+        self.assertEqual(
+            bern_gqs.sample_plus_quantities.shape[1],
+            bern_gqs.mcmc_sample.shape[1]
+            + bern_gqs.generated_quantities_pd.shape[1],
+        )
 
     def test_gen_quantities_csv_files_bad(self):
         stan = os.path.join(DATAFILES_PATH, 'bernoulli_ppc.stan')
@@ -62,17 +61,15 @@ class GenerateQuantitiesTest(unittest.TestCase):
         jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
 
         # synthesize list of filenames
-        goodfiles_path = os.path.join(DATAFILES_PATH, 'runset-bad',
-                                      'bad-draws-bern')
+        goodfiles_path = os.path.join(
+            DATAFILES_PATH, 'runset-bad', 'bad-draws-bern'
+        )
         csv_files = []
         for i in range(4):
-            csv_files.append('{}-{}.csv'.format(goodfiles_path, i+1))
+            csv_files.append('{}-{}.csv'.format(goodfiles_path, i + 1))
 
         with self.assertRaisesRegex(Exception, 'Invalid mcmc_sample'):
-            model.generate_quantities(
-                data=jdata,
-                mcmc_sample=csv_files
-                )
+            model.generate_quantities(data=jdata, mcmc_sample=csv_files)
 
     def test_gen_quanties_mcmc_sample(self):
         stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
@@ -80,16 +77,17 @@ class GenerateQuantitiesTest(unittest.TestCase):
 
         jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
         bern_fit = bern_model.sample(
-            data=jdata, chains=4, cores=2, seed=12345, iter_sampling=100
+            data=jdata,
+            chains=4,
+            parallel_chains=2,
+            seed=12345,
+            iter_sampling=100,
         )
 
         stan = os.path.join(DATAFILES_PATH, 'bernoulli_ppc.stan')
         model = CmdStanModel(stan_file=stan)
 
-        bern_gqs = model.generate_quantities(
-            data=jdata,
-            mcmc_sample=bern_fit
-        )
+        bern_gqs = model.generate_quantities(data=jdata, mcmc_sample=bern_fit)
         self.assertEqual(
             bern_gqs.runset._args.method, Method.GENERATE_QUANTITIES
         )
@@ -115,11 +113,14 @@ class GenerateQuantitiesTest(unittest.TestCase):
             'y_rep.10',
         ]
         self.assertEqual(bern_gqs.column_names, tuple(column_names))
-        self.assertEqual(bern_fit.get_drawset().shape,
-                         bern_gqs.mcmc_sample.shape)
-        self.assertEqual(bern_gqs.sample_plus_quantities.shape[1],
-                             bern_gqs.mcmc_sample.shape[1] +
-                             bern_gqs.generated_quantities_pd.shape[1])
+        self.assertEqual(
+            bern_fit.get_drawset().shape, bern_gqs.mcmc_sample.shape
+        )
+        self.assertEqual(
+            bern_gqs.sample_plus_quantities.shape[1],
+            bern_gqs.mcmc_sample.shape[1]
+            + bern_gqs.generated_quantities_pd.shape[1],
+        )
 
     def test_sample_plus_quantities_dedup(self):
         stan = os.path.join(DATAFILES_PATH, 'bernoulli_ppc.stan')
@@ -127,14 +128,17 @@ class GenerateQuantitiesTest(unittest.TestCase):
 
         jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
         bern_fit = model.sample(
-            data=jdata, chains=4, cores=2, seed=12345, iter_sampling=100
-        )
-        bern_gqs = model.generate_quantities(
             data=jdata,
-            mcmc_sample=bern_fit
+            chains=4,
+            parallel_chains=2,
+            seed=12345,
+            iter_sampling=100,
+        )
+        bern_gqs = model.generate_quantities(data=jdata, mcmc_sample=bern_fit)
+        self.assertEqual(
+            bern_gqs.sample_plus_quantities.shape[1],
+            bern_gqs.mcmc_sample.shape[1],
         )
-        self.assertEqual(bern_gqs.sample_plus_quantities.shape[1],
-                             bern_gqs.mcmc_sample.shape[1])
 
 
 if __name__ == '__main__':
